
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;Understanding the Dynamics of Violent Political Revolutions
;;;in an Agent-Based Framework
;;;
;;;written by
;;;
;;;ALESSANDRO MORO
;;;Ca' Foscari University, Venice
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;
;;DEFINITION OF AGENTS
;;;;;;;;;;;;;;;;;;;;;;;;;

breed [citizens citizen]
breed [policemen policeman]
breed [revolutionaries revolutionary]

globals [
  k1        ;arrest probability parameter
  k2        ;cost function parameter           
  Ey        ;expected value of income
  P         ;initial number of policemen
  R         ;initial number of revolutionaries
  t_rev     ;time of revolution
]

citizens-own [
  active?             ;if true, the citizen is rebelling
  jail-term           ;turns in jail (if 0, the citizen is not in jail)
  logy                ;logarithm of income
  y                   ;income
  hardship            ;economic deprivation H(y)
  grievance           ;political and economic grievance G(y)=(1-l)*H(y)
  cost                ;opportunity cost of rebelling J(y)
]

revolutionaries-own[
  in_action?          ;if true, the revolutionary is killing policemen
  ]


patches-own [
  neighborhood        ;surrounding patches within the vision radius
]


;;;;;;;;;;;;;;;;
;;SETUP
;;;;;;;;;;;;;;;;

to setup
  clear-all

  ;; set globals
  set k1 2.3       
  set k2 0.025     
  set Ey (exp( a + 0.5 * (b ^ 2) ))   
  set t_rev 999    
   
  ;random-seed 123 ;;uncomment to obtain the graphs of Figure 1
  ;random-seed 1234 ;;uncomment to obtain the simulations presented in the videos

  ask patches [
    set pcolor gray - 1
    set neighborhood patches in-radius vision
  ]

  ;; create policemen
  create-policemen round (initial-policeman-density * .01 * count patches) [
    move-to one-of patches with [not any? turtles-here]
    display-policeman
  ]
  
  ;; create revolutionaries
  create-revolutionaries round (initial-revolutionary-density * .01 * count patches) [
    move-to one-of patches with [not any? turtles-here]
    set in_action? false
    display-revolutionary
  ]

  ;; create citizens
  create-citizens round (initial-citizen-density * .01 * count patches) [
    move-to one-of patches with [not any? turtles-here]
    set heading 0
    set active? false
    set jail-term 0
    set logy random-normal a b
    set y (exp(logy))
    set hardship  (exp( Ey - y ) / (1 + exp( Ey - y ) ))
    set grievance (hardship * (1 - government-legitimacy))
    set cost (2 * (exp( k2 * max-jail-term * y )/(1 + exp(k2 * max-jail-term * y ))) - 1)
    display-citizen
  ]

  set P count policemen
  set R count revolutionaries

  ;; start clock and plot initial state of system
  reset-ticks
end


;;;;;;;;;;;;
;;GO
;;;;;;;;;;;;

to go
  
  ask turtles [
    ; Rule M: move to a random site within the vision radius
    if ((breed = citizens and jail-term = 0) or (breed = policemen) or (breed = revolutionaries)) [ move ]
    ; Rule C: determine if each citizen should be active or quiet
    if (breed = citizens and jail-term = 0) [ determine-behavior ]
    ; Rule R: determne if each revolutionary should be in action
    if (breed = revolutionaries) [ kill ]
    ; Rule P: arrest an active citizen or kill an active revolutionary
    if (breed = policemen) [ enforce ]
  ]
  
  ; Jailed citizens get their term reduced at the end of each clock tick
  ask citizens
    [ if (jail-term > 0) [ set jail-term (jail-term - 1) ] ]
    
  ; update agent display
  ask citizens [ display-citizen ]
  ask policemen [ display-policeman ]
  ask revolutionaries [ display-revolutionary ]
  
  ; advance clock and update plots
  tick
end


;;;;;;;;;;;;;;;;;;;
;; BEHAVIORS 
;;;;;;;;;;;;;;;;;;;


;; all turtles

to move 
    let targets neighborhood with   ;candidate patches are empty or contain only jailed citizens 
                [(not any? policemen-here) and (not any? revolutionaries-here) and (all? citizens-here [jail-term > 0])]
    if any? targets [ move-to one-of targets ]
end


;; citizens

to determine-behavior
  set active? (grievance - (estimated-arrest-probability * cost) > f )  ; active if and only if G(y)-A*J(y) > f
end


to-report estimated-arrest-probability 
  let P_v count (policemen-on neighborhood) 
  let C_v count (other citizens-on neighborhood) with [active?]
  let R_v count (revolutionaries-on neighborhood) with [in_action?]
  report (1 - exp (- k1 * floor (P_v / (1 + C_v + R_v))))
end


;; policemen

to enforce
  
  if (any? (turtles-on neighborhood) with [ ((breed = citizens) and active?) or ((breed = revolutionaries) and in_action?) ]) [
   ;select an active citizen or an active revolutionary
   let suspect one-of (turtles-on neighborhood) with [ ((breed = citizens) and active?) or ((breed = revolutionaries) and in_action?) ]
   
     if ([breed] of suspect = citizens) [ ;if the selected agent is a citizen, arrest him
       ask suspect [
       set active? false 
       set jail-term random max-jail-term
       ]
       move-to suspect ;move to the patch of the arrested citizen
       ]
     
     if ([breed] of suspect = revolutionaries)[ ;if the selected agent is a revolutionary, kill him
       let u_p random-float 1.0
       if (u_p <= policeman_precision)[ ;precision of policemen
       ask suspect [die]]
       ] 
     ]
end



;; revolutionaries

to kill
  
  if (((R + C) / P ) > n) [ ;active if and only if (R+C)/P > n
    
    set in_action? true
    if (ticks < t_rev)[set t_rev ticks] ;this command identifies the beginning of the revolution
    ]
  
  if ((in_action?) and (any? policemen-on neighborhood)) [ 
    ;select policeman
    let policeman_target one-of (policemen-on neighborhood) 
    let u_r random-float 1.0
    if (u_r <= revolutionary_precision)[ ;precision of revolutionaries
      ask policeman_target [die]
    ]
   
  ]
end

to-report C
  report count citizens with [active?]
end 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; VISUALIZATION OF AGENTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to display-citizen  
  set shape "circle"
  ifelse active?
    [ set color red ]  
  [ifelse jail-term > 0
        [ set color black + 3 ]  
        [ set color scale-color green grievance 1.5 -0.5 ] ]
end

to display-policeman
  set color cyan
  set shape "triangle"
end

to display-revolutionary
  set color orange
  set shape "square"
end
@#$#@#$#@
GRAPHICS-WINDOW
545
10
955
441
-1
-1
10.0
1
10
1
1
1
0
1
1
1
0
39
0
39
1
1
1
ticks
30.0

BUTTON
263
183
349
227
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
366
183
449
227
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
20
196
238
229
government-legitimacy
government-legitimacy
0.0
1.0
0.85
0.01
1
NIL
HORIZONTAL

SLIDER
20
237
239
270
max-jail-term
max-jail-term
0.0
50.0
30
1.0
1
turns
HORIZONTAL

MONITOR
436
315
511
360
active 
count citizens with [active?]
3
1
11

SLIDER
20
153
238
186
vision
vision
0.0
10.0
7
.1
1
patches
HORIZONTAL

MONITOR
349
316
429
361
jailed 
count citizens with [jail-term > 0]
1
1
11

SLIDER
17
67
235
100
initial-policeman-density
initial-policeman-density
0.0
100.0
4
1
1
%
HORIZONTAL

MONITOR
260
317
342
362
quiet
count citizens with [not active? and jail-term = 0]
1
1
11

MONITOR
350
259
428
304
policemen
count policemen
3
1
11

SLIDER
17
27
235
60
initial-citizen-density
initial-citizen-density
0.0
100.0
70
1.0
1
%
HORIZONTAL

MONITOR
262
258
340
303
citizens
count citizens
3
1
11

PLOT
373
445
955
762
All Agent Types
time
agents
0.0
20.0
0.0
150.0
true
true
"" ""
PENS
"jailed citizens" 1.0 0 -16777216 true "" "plot count citizens with [jail-term > 0]"
"active citizens" 1.0 0 -2674135 true "" "plot count citizens with [active?]"
"policemen" 1.0 0 -13345367 true "" "plot count policemen"
"revolutionaries" 1.0 0 -955883 true "" "plot count revolutionaries"

PLOT
20
445
180
594
Income
NIL
NIL
0.0
11.0
0.0
310.0
true
false
"" ""
PENS
"Income" 0.5 1 -16777216 true "" "histogram [y] of citizens"

PLOT
19
602
180
747
Grievance
NIL
NIL
0.0
0.15
0.0
200.0
true
false
"" ""
PENS
"default" 0.01 1 -16777216 true "" "histogram [grievance] of citizens"

SLIDER
18
110
235
143
initial-revolutionary-density
initial-revolutionary-density
0
100
3
1
1
%
HORIZONTAL

SLIDER
254
29
447
62
n
n
0
2
1.2
0.1
1
NIL
HORIZONTAL

MONITOR
435
257
520
302
revolutionaries
count revolutionaries
17
1
11

SLIDER
255
69
449
102
policeman_precision
policeman_precision
0
1
0.4
0.05
1
NIL
HORIZONTAL

SLIDER
255
113
450
146
revolutionary_precision
revolutionary_precision
0
1
0.3
0.05
1
NIL
HORIZONTAL

PLOT
183
602
359
747
Opportunity Cost
NIL
NIL
0.0
1.0
0.0
120.0
true
false
"" ""
PENS
"default" 0.05 1 -16777216 true "" "histogram [cost] of citizens"

SLIDER
18
278
242
311
a
a
-2
2
0.5
0.1
1
NIL
HORIZONTAL

SLIDER
19
363
242
396
f
f
0
1
0.1
0.01
1
NIL
HORIZONTAL

SLIDER
18
321
243
354
b
b
0
2
0.5
0.1
1
NIL
HORIZONTAL

PLOT
182
445
359
594
Hardship
NIL
NIL
0.0
1.0
0.0
150.0
true
false
"" ""
PENS
"default" 0.05 1 -16777216 true "" "histogram [hardship] of citizens"

@#$#@#$#@
## AUTHOR

ALESSANDRO MORO 
Ca' Foscari University, Venice. 
Email: alessandro.moro@unive.it


## WHAT IS IT?

This is the code that implements the model described in the paper "Understanding the Dynamics of Violent Political Revolutions in an Agent-Based Framework."
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

person active
false
0
Polygon -13791810 true false 135 90 150 105 135 165 150 180 165 165 150 105 165 90
Polygon -2674135 true false 195 135 240 30 210 15 165 120
Circle -7500403 true true 110 5 80
Rectangle -7500403 true true 127 79 172 94
Polygon -2674135 true false 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Polygon -2674135 true false 105 135 60 30 90 15 135 120
Polygon -6459832 true false 195 15 270 60 270 75 195 30

person jailed
false
0
Circle -7500403 true true 110 5 80
Polygon -16777216 true false 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -16777216 true false 195 90 210 150 195 180 165 105
Polygon -16777216 true false 105 90 90 150 105 180 135 105

person quiet
false
0
Polygon -13791810 true false 135 90 150 105 135 165 150 180 165 165 150 105 165 90
Polygon -1184463 true false 195 90 240 195 210 210 165 105
Circle -7500403 true true 110 5 80
Rectangle -7500403 true true 127 79 172 94
Polygon -1184463 true false 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Polygon -1 true false 100 210 130 225 145 165 85 135 63 189
Polygon -13791810 true false 90 210 120 225 135 165 67 130 53 189
Polygon -1 true false 120 224 131 225 124 210
Line -16777216 false 139 168 126 225
Line -16777216 false 140 167 76 136
Polygon -1184463 true false 105 90 60 195 90 210 135 105

person soldier
false
10
Rectangle -7500403 true false 127 79 172 94
Polygon -13345367 true true 105 90 60 195 90 210 135 105
Polygon -13345367 true true 195 90 240 195 210 210 165 105
Circle -7500403 true false 110 5 80
Polygon -13345367 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Polygon -6459832 true false 120 90 105 90 180 195 180 165
Line -6459832 false 109 105 139 105
Line -6459832 false 122 125 151 117
Line -6459832 false 137 143 159 134
Line -6459832 false 158 179 181 158
Line -6459832 false 146 160 169 146
Rectangle -6459832 true false 120 193 180 201
Polygon -6459832 true false 122 4 107 16 102 39 105 53 148 34 192 27 189 17 172 2 145 0
Polygon -16777216 true false 183 90 240 15 247 22 193 90
Rectangle -6459832 true false 114 187 128 208
Rectangle -6459832 true false 177 187 191 208

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 15 0 270 300 270

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.2.0
@#$#@#$#@
setup
repeat 5 [ go ]
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
